part of plot_window;

typedef num NumberFunction(num n);

class AverageMethod {
  String name;
  NumberFunction before;
  NumberFunction after;

  static Map<String, AverageMethod> CACHE = new Map<String, AverageMethod>();
  
  static AverageMethod ARITHMETIC = new AverageMethod._intern("Arithmetic",
      (num n)=>n, (num n)=>n);
    
  static AverageMethod GEOMETRIC = new AverageMethod._intern("Geometric",
      (num n)=>math.log(n), (num n)=>math.exp(n));

  bool operator ==(Object other) {
    if(!other is AverageMethod) return false;
    AverageMethod otherAverageMethod = other;
    return this.name==otherAverageMethod.name;
  }

  AverageMethod._intern(this.name, this.before, this.after) {
    CACHE[this.name] = this;
  }
  
  static init() {
    ARITHMETIC; GEOMETRIC;
  }

}


class PointSmoother {
  AverageMethod average;
  int width;

  String toString()=>"${this.average.name} (${this.width})";
  
  bool operator ==(Object other) {
    if(!other is PointSmoother) return false;
    PointSmoother otherPointSmoother = other;
    return this.average==otherPointSmoother.average && this.width==otherPointSmoother.width;
  }

  PointSmoother(this.average, this.width);

  List<Point> smooth(Iterable<Point> pointsIterable) {
    List<Point> points = pointsIterable.toList();
    if (points.length < 2 * width + 1) { throw "points list too short"; }

    List<num> input = points.map((Point p)=>average.before(p.y)/(2*width+1)).toList();
    var vals = new List<num>(points.length - 2*width);
    vals[0] = input.take(2*width +1).fold(0.0, (num total, num elem )=>total+elem);
    for (int i=1; i<vals.length; i++) {
      vals[i] = vals[i-1] - input[i-1] + input[i+2*width];
    }

    var ret = new List<Point>(vals.length);
    for (int i=0; i<vals.length; i++) {
      ret[i] = new Point(points[width+i].x, average.after(vals[i]));
    }
    return ret;
  }
  
}