part of plot_window;

abstract class Settable {
  Iterable<Setting> get settings;
}

class SettingsGroup {
  List<Setting> settings;

  DivElement container;
  
  void perform(MouseEvent e) {
    settings.forEach((Setting s)=>s.perform());
    hide();
  }
  
  SettingsGroup() {
    settings = new List<Setting>();
    container = new DivElement();
    container.style..display="none"
      ..position="fixed"
      ..width="60vw"
      ..height="40vh"
      ..marginTop="20vh"
      ..marginLeft="20vw"
      ..padding="5vw"
      ..top="0"
      ..left="0"
      ..backgroundColor="white"
      ..textAlign="middle"
      ..boxSizing="border-box";
    
  }

  void show() {
    print("displaying settings group");
    container.style.display="block";
  }

  void hide() {
    container.style.display="none";
  }

  void populate(Settable settable, [bool reset=true]) {
    print("populate called with $settable");
    if(reset)
      this.settings.clear();
    this.settings.addAll(settable.settings);
  }

  void addSetting(Setting setting) {
    this.settings.add(setting);
  }

  Future run() async {
    print("run called");
    container.children.clear();
    container.children.addAll(settings.map((Setting s)=>s.html));
    var b = new ButtonElement()..text="Done";
    container.children.add(b);
    show();
    return await b.onClick.first.then(perform);
  }

}


typedef void Action(dynamic);

abstract class Setting {
  String text;
  Action action;
  InputElement inputElement;

  Setting(this.text, this.action);
  
  Element get html {
    DivElement div = new DivElement();
    div.children..add(new LabelElement()..text=text)
    ..add(inputElement);

    return div;
  }

  
  void perform();
}

class StringSetting extends Setting {
  
  String value;
  
  StringSetting(String text, this.value, Action action):super(text, action) {
    this.inputElement = new InputElement()..value=value;
  }

  void perform() {
    if(inputElement.value!=value) {
      action(inputElement.value);
    }
  }
}

class NumSetting extends Setting {
  num value;
  
  NumSetting(String text, this.value, Action action):super(text, action) {
    this.inputElement = new NumberInputElement()..valueAsNumber=value;
  }

  void perform() {
    if(inputElement.valueAsNumber!=value) {
      action(inputElement.valueAsNumber);
    }
  }
}

class DataListSetting extends Setting {
  List values;
  DataListElement dle;
  var current;
  String id;

  DataListSetting(String text, this.current, this.values, Action action):super(text, action) {
    id = values.map((var value)=>"$value").join("_");
    dle = new DataListElement()..id=id;
    dle.children.addAll(values.map((var val)=>new OptionElement()..value="$val"));
    this.inputElement = new TextInputElement()..value="${current}";
  }

  Element get html {
    DivElement div = new DivElement();
    div.children..add(new LabelElement()..text=text)
    ..addAll([inputElement, dle]);
    inputElement.setAttribute("list", id);
    return div;
  }

  void perform() {
    action(inputElement.value);
  }
}


class BoolSetting extends Setting {
  bool value;

  BoolSetting(String text, this.value, Action action):super(text, action) {
    this.inputElement = new CheckboxInputElement()..checked=value;
  }

  void perform() {
    action(inputElement.checked);
  }
}


class ColorSetting extends Setting {

  String value;
  
  ColorSetting(String text, this.value, Action action):super(text, action) {
    this.inputElement = new InputElement()..value=value
      ..type="color";
  }
    
  void perform() {
    if(inputElement.value!=value) {
      action(inputElement.value);
    }
  }
  
}